//+------------------------------------------------------------------+
//|                                             Indicator Arrows.mq4 |
//|                        Copyright 2014, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2014, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
#property strict
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 clrLimeGreen
#property indicator_color2 clrFireBrick
//---
enum indicators
  {
   INDICATOR_MA,           //Moving Average
   INDICATOR_MACD,         //Moving Average Convergence/Divergence
   INDICATOR_OSMA,         //Oscillator of Moving Averages
   INDICATOR_STOCHASTIC,   //Stochastic Oscillator
   INDICATOR_RSI,          //Relative Strength Index
   INDICATOR_CCI,          //Commodity Channel Index
   INDICATOR_RVI,          //Relative Vigor Index
   INDICATOR_ADX,          //Average Directional Movement Index
   INDICATOR_BANDS,        //Bollinger Bands
   INDICATOR_NONE          //No Indicator
  };
//---
input indicators Indicator1=INDICATOR_MACD;
input ENUM_TIMEFRAMES TimeFrame1=0;
input indicators Indicator2=INDICATOR_MA;
input ENUM_TIMEFRAMES TimeFrame2=0;
//---Range
input string Range;
input int RangePeriod=14;
//---Moving Average
input string MovingAverage;
input int MAPeriod=5;
input ENUM_MA_METHOD MAMethod=MODE_EMA;
input ENUM_APPLIED_PRICE MAPrice=PRICE_CLOSE;
//---MACD
input string MACD;
input int FastMACD=12;
input int SlowMACD=26;
input int SignalMACD=9;
input ENUM_APPLIED_PRICE MACDPrice=PRICE_CLOSE;
//---OsMA
input string OsMA;
input int FastOsMA=12;
input int SlowOsMA=26;
input int SignalOsMA=9;
input ENUM_APPLIED_PRICE OsMAPrice=PRICE_CLOSE;
input int OsMASignal=9;
//---Stoch
input string Stochastic;
input int Kperiod=8;
input int Dperiod=3;
input int Slowing=3;
input ENUM_STO_PRICE PriceField=STO_LOWHIGH;
//---RSI
input string RSI;
input int RSIPeriod=8;
input int RSISignal=3;
input ENUM_APPLIED_PRICE RSIPrice=PRICE_CLOSE;
//---CCI
input string CCI;
input int CCIPeriod=14;
input int CCISignal=3;
input ENUM_APPLIED_PRICE CCIPrice=PRICE_CLOSE;
//---RVI
input string RVI;
input int RVIPeriod=10;
//---ADX
input string ADX;
input int ADXPeriod=14;
input ENUM_APPLIED_PRICE ADXPrice=PRICE_CLOSE;
//---Bands
input string Bands;
input int BBPeriod=20;  //Bands Period
input double BBDev=2.0; //Bands Deviation
input ENUM_APPLIED_PRICE BBPrice=PRICE_CLOSE;   //Bands Price
input string _;//---
//---Alerts
input bool  AlertsOn      = true,
            AlertsMessage = true,
            AlertsEmail   = false,
            AlertsSound   = false;
//---
int indicator1,
    indicator2;
//---
double Buy[];
double Sell[];
//---
string name1="LabelBox",
       name2="Label1",
       name3="Label2",
       name4="Label3",
       name5="Label4",
       label1="Spread ",
       label2="Range(";
//---
int doubleToPip;
double pipToDouble;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- set points & digits
   if(_Digits==2 || _Digits==3)  
      doubleToPip = 100;
   else                          
      doubleToPip = 10000;
   
   if(_Digits==2 || _Digits==4) 
      pipToDouble=_Point;
   else                       
      pipToDouble=_Point*10;
//--- indicator buffers mapping
   IndicatorBuffers(indicator_buffers);
   SetIndexStyle(0,DRAW_ARROW);
   SetIndexArrow(0,233);
   SetIndexStyle(1,DRAW_ARROW);
   SetIndexArrow(1,234);
   SetIndexBuffer(0,Buy);
   SetIndexBuffer(1,Sell);
//---
   string timeFrame1,
          timeFrame2;
   switch(TimeFrame1)
     {
      case PERIOD_MN1: timeFrame1 = "MN1 ";break;
      case PERIOD_W1:  timeFrame1 = "W1 "; break;
      case PERIOD_D1:  timeFrame1 = "D1 "; break;
      case PERIOD_H4:  timeFrame1 = "H4 "; break;
      case PERIOD_H1:  timeFrame1 = "H1 "; break;
      case PERIOD_M30: timeFrame1 = "M30 ";break;
      case PERIOD_M15: timeFrame1 = "M15 ";break;
      case PERIOD_M5:  timeFrame1 = "M5 "; break;
      case PERIOD_M1:  timeFrame1 = "M1 "; break;
      default:         timeFrame1 = "";
     }   
   switch(TimeFrame2)
     {
      case PERIOD_MN1: timeFrame2 = "MN1 ";break;
      case PERIOD_W1:  timeFrame2 = "W1 "; break;
      case PERIOD_D1:  timeFrame2 = "D1 "; break;
      case PERIOD_H4:  timeFrame2 = "H4 "; break;
      case PERIOD_H1:  timeFrame2 = "H1 "; break;
      case PERIOD_M30: timeFrame2 = "M30 ";break;
      case PERIOD_M15: timeFrame2 = "M15 ";break;
      case PERIOD_M5:  timeFrame2 = "M5 "; break;
      case PERIOD_M1:  timeFrame2 = "M1 "; break;
      default:         timeFrame2 = "";
     }
//---create label rectangle and labels
   string label3,
          label4;
   int xStart=7;
   int yStart=80;
   int yIncrement=14;
   int ySize=40;
   int ySizeInc=15;
   ObjectCreate(name1,OBJ_RECTANGLE_LABEL,0,0,0);
   ObjectSet(name1,OBJPROP_CORNER,0);
   ObjectSet(name1,OBJPROP_XDISTANCE,3);
   ObjectSet(name1,OBJPROP_YDISTANCE,75);
   ObjectSet(name1,OBJPROP_XSIZE,135);
   ObjectSet(name1,OBJPROP_YSIZE,ySize);
   ObjectSet(name1,OBJPROP_BGCOLOR,clrBlack);
   ObjectSet(name1,OBJPROP_BORDER_TYPE,BORDER_FLAT);

   ObjectCreate(name2,OBJ_LABEL,0,0,0);
   ObjectSet(name2,OBJPROP_CORNER,0);
   ObjectSet(name2,OBJPROP_XDISTANCE,xStart);
   ObjectSet(name2,OBJPROP_YDISTANCE,yStart);
   
   ObjectCreate(name3,OBJ_LABEL,0,0,0);
   ObjectSet(name3,OBJPROP_CORNER,0);
   ObjectSet(name3,OBJPROP_XDISTANCE,xStart);
   ObjectSet(name3,OBJPROP_YDISTANCE,yStart+=yIncrement);
//---
   if(Indicator1==INDICATOR_NONE)
      Alert("Indicator1 can't be 'No Indicator'");
   switch(Indicator1)
     {
      case INDICATOR_MA:
         label3 = "iMA "+timeFrame1+
                  StringSubstr(EnumToString(MAMethod),5)+" ("+
                  IntegerToString(MAPeriod)+")";
         break;
      case INDICATOR_MACD:
         label3 = "iMACD "+timeFrame1+"("+
                  IntegerToString(FastMACD)+","+
                  IntegerToString(SlowMACD)+","+
                  IntegerToString(SignalMACD)+")";
         break;
      case INDICATOR_OSMA:
         label3 = "iOsMA "+timeFrame1+"("+
                  IntegerToString(FastOsMA)+","+
                  IntegerToString(SlowOsMA)+","+
                  IntegerToString(SignalOsMA)+")";
         break;
      case INDICATOR_STOCHASTIC:
         label3 = "iStoch "+timeFrame1+"("+
                  IntegerToString(Kperiod)+","+
                  IntegerToString(Dperiod)+","+
                  IntegerToString(Slowing)+")";
         break;
      case INDICATOR_RSI:
         label3 = "iRSI "+timeFrame1+"("+
                  IntegerToString(RSIPeriod)+","+
                  IntegerToString(RSISignal)+")";
         break;
      case INDICATOR_CCI:
         label3 = "iCCI "+timeFrame1+"("+
                  IntegerToString(CCIPeriod)+","+
                  IntegerToString(CCISignal)+")";
         break;
      case INDICATOR_RVI:
         label3 = "iRVI "+timeFrame1+"("+
                  IntegerToString(RVIPeriod)+")";
         break;
      case INDICATOR_ADX:
         label3 = "iADX "+timeFrame1+"("+
                  IntegerToString(ADXPeriod)+")";
         break;
      case INDICATOR_BANDS:
         label3 = "iBands "+timeFrame1+"("+
                  IntegerToString(BBPeriod)+","+
                  DoubleToString(BBDev,1)+")";
     }
   ObjectSet(name1,OBJPROP_YSIZE,ySize+=ySizeInc);
   ObjectCreate(name4, OBJ_LABEL,0,0,0);
   ObjectSet(name4,OBJPROP_CORNER,0);
   ObjectSet(name4,OBJPROP_XDISTANCE,xStart);
   ObjectSet(name4,OBJPROP_YDISTANCE,yStart+=yIncrement);
   ObjectSetText(name4,label3,10,"Arial Bold",clrLimeGreen);
//---
   if(Indicator2!=INDICATOR_NONE)
     {
      switch(Indicator2)
        {
         case INDICATOR_MA:
            label4 = "iMA "+timeFrame2+
                     StringSubstr(EnumToString(MAMethod),5)+" ("+
                     IntegerToString(MAPeriod)+")";
            break;
         case INDICATOR_MACD:
            label4 = "iMACD "+timeFrame2+"("+
                     IntegerToString(FastMACD)+","+
                     IntegerToString(SlowMACD)+","+
                     IntegerToString(SignalMACD)+")";
            break;
         case INDICATOR_OSMA:
            label4 = "iOsMA "+timeFrame2+"("+
                     IntegerToString(FastOsMA)+","+
                     IntegerToString(SlowOsMA)+","+
                     IntegerToString(SignalOsMA)+")";
            break;
         case INDICATOR_STOCHASTIC:
            label4 = "iStoch "+timeFrame2+"("+
                     IntegerToString(Kperiod)+","+
                     IntegerToString(Dperiod)+","+
                     IntegerToString(Slowing)+")";
            break;
         case INDICATOR_RSI:
            label4 = "iRSI "+timeFrame2+"("+
                     IntegerToString(RSIPeriod)+","+
                     IntegerToString(RSISignal)+")";
            break;
         case INDICATOR_CCI:
            label4 = "iCCI "+timeFrame2+"("+
                     IntegerToString(CCIPeriod)+","+
                     IntegerToString(CCISignal)+")";
            break;
         case INDICATOR_RVI:
            label4 = "iRVI "+timeFrame2+"("+
                     IntegerToString(RVIPeriod)+")";
            break;
         case INDICATOR_ADX:
            label4 = "iADX "+timeFrame2+"("+
                     IntegerToString(ADXPeriod)+")";
            break;
         case INDICATOR_BANDS:
            label4 = "iBands "+timeFrame2+"("+
                     IntegerToString(BBPeriod)+","+
                     DoubleToString(BBDev,1)+")";
        }      
      ObjectSet(name1,OBJPROP_YSIZE,ySize+=ySizeInc);
      ObjectCreate(name5,OBJ_LABEL,0,0,0);
      ObjectSet(name5,OBJPROP_CORNER, 0);
      ObjectSet(name5,OBJPROP_XDISTANCE,xStart);
      ObjectSet(name5,OBJPROP_YDISTANCE,yStart+=yIncrement);
      ObjectSetText(name5,label4,10,"Arial Bold",clrLimeGreen);
     }
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
#define OP_NONE -1
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   if(rates_total<100)  
      return(0);
   ObjectSetText(name2,label1+
                 DoubleToStr((Ask-Bid)*doubleToPip,1),10,"Arial Bold",clrFireBrick);
   static datetime prevTime;
   if(prev_calculated>0 && time[0]!=prevTime)
     {
      prevTime = time[0];
      ObjectSetText(name3,label2+string(RangePeriod)+") "+
                    DoubleToStr(Range(0)*doubleToPip,1),10,"Arial Bold",clrYellow);
     }
   int i,limit,shift1,shift2;
//---
   limit = rates_total - prev_calculated;
   if(prev_calculated==0)
      limit -= 100;
//---
   for(i=limit;i>=0;i--)
     {
      shift1 = iBarShift(_Symbol,TimeFrame1,time[i]);
      switch(Indicator1)
        {
         case INDICATOR_MA:
            indicator1 = iMA(TimeFrame1,shift1);
            break;
         case INDICATOR_MACD:
            indicator1 = iMACD(TimeFrame1,shift1);
            break;
         case INDICATOR_OSMA:
            indicator1 = iOsMA(TimeFrame1,shift1);
            break;
         case INDICATOR_STOCHASTIC:
            indicator1 = iStochastic(TimeFrame1,shift1);
            break;
         case INDICATOR_RSI:
            indicator1 = iRSI(TimeFrame1,shift1);
            break;
         case INDICATOR_CCI:
            indicator1 = iCCI(TimeFrame1,shift1);
            break;
         case INDICATOR_RVI:
            indicator1 = iRVI(TimeFrame1,shift1);
            break;
         case INDICATOR_ADX:
            indicator1 = iADX(TimeFrame1,shift1);
            break;
         case INDICATOR_BANDS:
            indicator1 = iBands(TimeFrame1,shift1);
        }

      shift2 = iBarShift(_Symbol,TimeFrame2,time[i]);
      switch(Indicator2)
        {
         case INDICATOR_MA:
            indicator2 = iMA(TimeFrame2,shift2);
            break;
         case INDICATOR_MACD:
            indicator2 = iMACD(TimeFrame2,shift2);
            break;
         case INDICATOR_OSMA:
            indicator2 = iOsMA(TimeFrame2,shift2);
            break;
         case INDICATOR_STOCHASTIC:
            indicator2 = iStochastic(TimeFrame2,shift2);
            break;
         case INDICATOR_RSI:
            indicator2 = iRSI(TimeFrame2,shift2);
            break;
         case INDICATOR_CCI:
            indicator2 = iCCI(TimeFrame2,shift2);
            break;
         case INDICATOR_RVI:
            indicator2 = iRVI(TimeFrame2,shift2);
            break;
         case INDICATOR_ADX:
            indicator2 = iADX(TimeFrame2,shift2);
            break;
         case INDICATOR_BANDS:
            indicator2 = iBands(TimeFrame2,shift2);
         case INDICATOR_NONE:
            break;
        }

      if(Indicator2==INDICATOR_NONE)
        {
         if(indicator1==OP_BUY)
           {
            Buy[i] = Low[i] - Range(i);
            if(AlertsOn && prev_calculated>0) 
               AlertsHandle(time[0],OP_BUY);
            Sell[i] = EMPTY_VALUE;
           }
         else if(indicator1==OP_SELL)   
           {
            Sell[i] = High[i] + Range(i);
            if(AlertsOn && prev_calculated>0) 
               AlertsHandle(time[0],OP_SELL);
            Buy[i] = EMPTY_VALUE;
           }
         else
           {
            Buy[i] = EMPTY_VALUE;
            Sell[i]= EMPTY_VALUE;
           } 
        }
      else
        {
         if(indicator1==OP_BUY && indicator2==OP_BUY)
           {
            Buy[i] = Low[i] - Range(i);
            if(AlertsOn && prev_calculated>0) 
               AlertsHandle(time[0],OP_BUY);
            Sell[i] = EMPTY_VALUE;
           }
         else if(indicator1==OP_SELL && indicator2==OP_SELL)   
           {
            Sell[i] = High[i] + Range(i);
            if(AlertsOn && prev_calculated>0) 
               AlertsHandle(time[0],OP_SELL);
            Buy[i] = EMPTY_VALUE;
           }
         else
           {
            Buy[i] = EMPTY_VALUE;
            Sell[i]= EMPTY_VALUE;
           } 
        }
     }
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---check & delete labels
   if(ObjectFind(0,name1)!=-1)
      ObjectDelete(0,name1);
   if(ObjectFind(0,name2)!=-1)
      ObjectDelete(0,name2);
   if(ObjectFind(0,name3)!=-1)
      ObjectDelete(0,name3);
   if(ObjectFind(0,name4)!=-1)
      ObjectDelete(0,name4);
   if(ObjectFind(0,name5)!=-1)
      ObjectDelete(0,name5);
   return;
  }
//+------------------------------------------------------------------+
//|  Average Range                                                   |
//+------------------------------------------------------------------+
double Range(int idx)
  {
   double avgRange = iATR(_Symbol,0,RangePeriod,idx+1);
   return(avgRange);
  }
//+------------------------------------------------------------------+
//|  Moving Average                                                  |
//+------------------------------------------------------------------+
int iMA(const ENUM_TIMEFRAMES timeframe,
        const int idx)
  { 
   int signal=OP_NONE;
   double currMA = iMA(_Symbol,timeframe,MAPeriod,0,MAMethod,MAPrice,idx);
   double prevMA = iMA(_Symbol,timeframe,MAPeriod,0,MAMethod,MAPrice,idx+1);
   
   if(currMA>prevMA+1*pipToDouble)
      signal=OP_BUY;
   if(currMA<prevMA-1*pipToDouble)
      signal=OP_SELL;

   return(signal);
  }
//+------------------------------------------------------------------+
//|  Moving Average Convergence/Divergence                           |
//+------------------------------------------------------------------+
int iMACD(const ENUM_TIMEFRAMES timeframe,
          const int idx)
  { 
   int signal=OP_NONE;
   double currMACD = iMACD(_Symbol,timeframe,FastMACD,SlowMACD,SignalMACD,MACDPrice,MODE_MAIN,idx);
   double currSign = iMACD(_Symbol,timeframe,FastMACD,SlowMACD,SignalMACD,MACDPrice,MODE_SIGNAL,idx);
   double prevMACD = iMACD(_Symbol,timeframe,FastMACD,SlowMACD,SignalMACD,MACDPrice,MODE_MAIN,idx+1);
   double prevSign = iMACD(_Symbol,timeframe,FastMACD,SlowMACD,SignalMACD,MACDPrice,MODE_SIGNAL,idx+1);
   
   if(currMACD>currSign && prevMACD<prevSign)
      signal=OP_BUY;
   if(currMACD<currSign && prevMACD>prevSign)
      signal=OP_SELL;

   return(signal);
  }
//+------------------------------------------------------------------+
//|  Oscillator of Moving Averages                                   |
//+------------------------------------------------------------------+
int iOsMA(const ENUM_TIMEFRAMES timeframe,
          const int idx)
  { 
   int signal=OP_NONE;
   double currOsMA = iOsMA(_Symbol,timeframe,FastOsMA,SlowOsMA,SignalOsMA,OsMAPrice,OsMASignal,MODE_MAIN,idx);
   double currSign = iOsMA(_Symbol,timeframe,FastOsMA,SlowOsMA,SignalOsMA,OsMAPrice,OsMASignal,MODE_SIGNAL,idx);
   double prevOsMA = iOsMA(_Symbol,timeframe,FastOsMA,SlowOsMA,SignalOsMA,OsMAPrice,OsMASignal,MODE_MAIN,idx+1);
   double prevSign = iOsMA(_Symbol,timeframe,FastOsMA,SlowOsMA,SignalOsMA,OsMAPrice,OsMASignal,MODE_SIGNAL,idx+1);

   if(currOsMA>currSign && prevOsMA<prevSign)
      signal=OP_BUY;
   if(currOsMA<currSign && prevOsMA>prevSign)
      signal=OP_SELL;

   return(signal);
  }
//---
double iOsMA(const string symbol,
             const ENUM_TIMEFRAMES timeframe,
             int fast_ema_period,
             int slow_ema_period,
             int signal_period,
             const ENUM_APPLIED_PRICE price,
             int signal,
             int mode,
             int idx)
  {
   double osma;
   if(mode==MODE_SIGNAL)
     {
      double sum=0.0;
      static double osmaSignal[1];
      if(ArraySize(osmaSignal)!=signal)
         ArrayResize(osmaSignal,signal);
      for(int i=0;i<signal;i++)
        {
         osmaSignal[i]=iOsMA(symbol,
                             timeframe,
                             fast_ema_period,
                             slow_ema_period,
                             signal_period,
                             price,
                             idx+i);
         sum+=osmaSignal[i];
        }
      osma=sum/signal;
     }
   else
      osma=iOsMA(symbol,
                 timeframe,
                 fast_ema_period,
                 slow_ema_period,
                 signal_period,
                 price,
                 idx);
   return(osma);
  }
//+------------------------------------------------------------------+
//|  Stochastic Oscillator                                           |
//+------------------------------------------------------------------+
int iStochastic(const ENUM_TIMEFRAMES timeframe,
                const int idx)
  { 
   int signal=OP_NONE;
   double currStoc = iStochastic(_Symbol,timeframe,Kperiod,Dperiod,Slowing,MODE_SMA,PriceField,MODE_MAIN,idx);
   double currSign = iStochastic(_Symbol,timeframe,Kperiod,Dperiod,Slowing,MODE_SMA,PriceField,MODE_SIGNAL,idx);
   double prevStoc = iStochastic(_Symbol,timeframe,Kperiod,Dperiod,Slowing,MODE_SMA,PriceField,MODE_MAIN,idx+1);
   double prevSign = iStochastic(_Symbol,timeframe,Kperiod,Dperiod,Slowing,MODE_SMA,PriceField,MODE_SIGNAL,idx+1);
   
   if(currStoc>currSign && prevStoc<prevSign)
      signal=OP_BUY;
   if(currStoc<currSign && prevStoc>prevSign)
      signal=OP_SELL;

   return(signal);
  }            
//+------------------------------------------------------------------+
//|  Relative Strength Index                                         |
//+------------------------------------------------------------------+
int iRSI(const ENUM_TIMEFRAMES timeframe,
         const int idx)
  { 
   int signal=OP_NONE;
   double currRsi = iRSI(_Symbol,timeframe,RSIPeriod,RSISignal,RSIPrice,MODE_MAIN,idx);
   double currSign= iRSI(_Symbol,timeframe,RSIPeriod,RSISignal,RSIPrice,MODE_SIGNAL,idx);
   double prevRsi = iRSI(_Symbol,timeframe,RSIPeriod,RSISignal,RSIPrice,MODE_MAIN,idx+1);
   double prevSign= iRSI(_Symbol,timeframe,RSIPeriod,RSISignal,RSIPrice,MODE_SIGNAL,idx+1);
   
   if(currRsi>currSign && prevRsi<prevSign)
      signal=OP_BUY;
   if(currRsi<currSign && prevRsi>prevSign)
      signal=OP_SELL;
 
   return(signal);
  }            
//---
double iRSI(const string symbol,
            const ENUM_TIMEFRAMES timeframe,
            const int period,
            const int signal,
            const ENUM_APPLIED_PRICE price,
            const int mode=MODE_MAIN,
            const int idx=0)
  {
   double rsi;
   if(mode==MODE_SIGNAL)
     {
      double sum=0.0;
      static double rsiSignal[];
      if(ArraySize(rsiSignal)!=signal)
         ArrayResize(rsiSignal,signal);
      for(int i=0;i<signal;i++)
        {
         rsiSignal[i]=iRSI(symbol,timeframe,period,price,idx+i);
         sum+=rsiSignal[i];
        }
      rsi=sum/signal;
     }
   else
      rsi=iRSI(symbol,timeframe,period,price,idx);
//---
   return(rsi);
  }
//+------------------------------------------------------------------+
//|  Commodity Channel Index                                         |
//+------------------------------------------------------------------+
int iCCI(const ENUM_TIMEFRAMES timeframe,
         const int idx)
  { 
   int signal=OP_NONE;
   double currCci = iCCI(_Symbol,timeframe,CCIPeriod,CCISignal,CCIPrice,MODE_MAIN,idx);
   //double currSign = iCCI(_Symbol,timeframe,CCIPeriod,CCISignal,CCIPrice,MODE_SIGNAL,idx);
   double prevCci = iCCI(_Symbol,timeframe,CCIPeriod,CCISignal,CCIPrice,MODE_MAIN,idx+1);
   //double prevSign = iCCI(_Symbol,timeframe,CCIPeriod,CCISignal,CCIPrice,MODE_SIGNAL,idx+1);
   double level[3] = {-100,0,100};
   
   for(int i=0;i<3;i++)
     {
      if(currCci>level[i] && prevCci<=level[i])
        {
         signal=OP_BUY;
         break;
        } 
      if(currCci<level[i] && prevCci>=level[i])
        {
         signal=OP_SELL;
         break;
        } 
     } 

   return(signal);
  }            
//---
double iCCI(const string symbol,
            const ENUM_TIMEFRAMES timeframe,
            const int period,
            const int signal,
            const ENUM_APPLIED_PRICE price,
            const int mode=MODE_MAIN,
            const int idx=0)
  {
   double cci;
   if(mode==MODE_SIGNAL)
     {
      double sum=0.0;
      static double cciSignal[];
      if(ArraySize(cciSignal)!=signal)
         ArrayResize(cciSignal,signal);
      for(int i=0;i<signal;i++)
        {
         cciSignal[i]=iCCI(symbol,timeframe,period,price,idx+i);
         sum+=cciSignal[i];
        }
      cci=sum/signal;
     }
   else
      cci=iCCI(symbol,timeframe,period,price,idx);
//---
   return(cci);
  }
//+------------------------------------------------------------------+
//|  Relative Vigor Index                                            |
//+------------------------------------------------------------------+
int iRVI(const ENUM_TIMEFRAMES timeframe,
         const int idx)
  { 
   int signal=OP_NONE;
   double currRvi = iRVI(_Symbol,timeframe,RVIPeriod,MODE_MAIN,idx);
   double currSign= iRVI(_Symbol,timeframe,RVIPeriod,MODE_SIGNAL,idx);
   double prevRvi = iRVI(_Symbol,timeframe,RVIPeriod,MODE_MAIN,idx+1);
   double prevSign= iRVI(_Symbol,timeframe,RVIPeriod,MODE_SIGNAL,idx+1);
   
   if(currRvi>currSign && prevRvi<prevSign)
      signal=OP_BUY;
   if(currRvi<currSign && prevRvi>prevSign)
      signal=OP_SELL;
 
   return(signal);
  }            
//+------------------------------------------------------------------+
//|  Average Directional Movement Index                              |
//+------------------------------------------------------------------+
int iADX(const ENUM_TIMEFRAMES timeframe,
         const int idx)
  { 
   int signal=OP_NONE;
   bool adx = iADX(_Symbol,timeframe,ADXPeriod,ADXPrice,MODE_MAIN,idx)<25;
   double currPDi = iADX(_Symbol,timeframe,ADXPeriod,ADXPrice,MODE_PLUSDI,idx);
   double currMDi = iADX(_Symbol,timeframe,ADXPeriod,ADXPrice,MODE_MINUSDI,idx);
   double prevPDi = iADX(_Symbol,timeframe,ADXPeriod,ADXPrice,MODE_PLUSDI,idx+1);
   double prevMDi = iADX(_Symbol,timeframe,ADXPeriod,ADXPrice,MODE_MINUSDI,idx+1);
   
   if(adx && currPDi>currMDi && prevPDi<prevMDi)
      signal=OP_BUY;
   if(adx && currPDi<currMDi && prevPDi>prevMDi)
      signal=OP_SELL;
//---
   return(signal);
  }            
//+------------------------------------------------------------------+
//|  Bollinger Bands                                                 |
//+------------------------------------------------------------------+
int iBands(const ENUM_TIMEFRAMES timeframe,
           const int idx)
  { 
   int signal=OP_NONE;
   double high = iHigh(_Symbol,timeframe,idx);
   double low = iLow(_Symbol,timeframe,idx);
   double close = iClose(_Symbol,timeframe,idx);
   double upperBand = iBands(_Symbol,timeframe,BBPeriod,BBDev,0,BBPrice,MODE_UPPER,idx);
   double midleBand = iBands(_Symbol,timeframe,BBPeriod,BBDev,0,BBPrice,MODE_MAIN,idx);
   double lowerBand = iBands(_Symbol,timeframe,BBPeriod,BBDev,0,BBPrice,MODE_LOWER,idx);
   
   if((low<lowerBand && close>lowerBand) ||
      (low<midleBand && close>midleBand))
      signal=OP_BUY;
   if((high>upperBand && close<upperBand) ||
      (high>midleBand && close<midleBand))
      signal=OP_SELL;
//---
   return(signal);
  }               
//+------------------------------------------------------------------+
//|  Alerts Handle                                                   |
//+------------------------------------------------------------------+
void AlertsHandle(const datetime &time,const ENUM_ORDER_TYPE alert_type)
  {
   static datetime timePrev;
   static ENUM_ORDER_TYPE typePrev;
   string alertMessage;

   if(timePrev!=time || typePrev!=alert_type)
     {
      timePrev=time;
      typePrev=alert_type;
      alertMessage=StringConcatenate(Symbol()," @ ",TimeToStr(TimeLocal(),TIME_MINUTES)," ",
                                     StringSubstr(EnumToString(alert_type),11)," @ ",DoubleToStr(Bid,_Digits));
      if(AlertsMessage) 
         Alert(alertMessage);
      if(AlertsEmail) 
         SendMail(StringConcatenate(Symbol(),"Arrow Alert "),alertMessage);
      if(AlertsSound) 
         PlaySound("alert2.wav");
     }
  }
//+------------------------------------------------------------------+
